
function [Meas_mtx Meas_mtx_noise]=gen_data_noise(original_network,B_vct,time_points,perc_noise,plot_data,x0)

% This function generates the state response, taking a certain number of
% points (time_points), of a stable dynamical linear system
% (dot(x)= Ax +Bu), whose A (dynamic matrix) is original_network and
% the B vector is B_vec.
%
% INPUT VARIABLES
%
% original_network:  the dynamic matrix of the stable dynamic linear system
%
% B_vct:             the vector that models the effect of external
%                    perturbations
% 
% time_points:       number of time points of the state response 
%
% perc_noise:        fraction of noise level
%
% plot_data:         if true, plot the data
%
% x0:                initial condition of the states 
%
% OUTPUT VARIABLES
%   
% Meas_mtx          Measurements matrix without noise, each column contains 
%                   the measurements for a node
%
% Meas_mtx_noise    Measurements matrix with the fixed level of noise, each   
%                   column contains the measurements for a node

% number of nodes of the network
n_nodes=size(original_network,1);

if nargin==5
    % Default value of xo
    x0=-1 +2.*rand(n_nodes,1);
    
elseif nargin~=6
    % Wrong number of input arguments
    error('Wrong number of input arguments');
end



% number of nodes of the network
n_nodes=size(original_network,1);

% generate the stable linear system
sys=ss(original_network,B_vct,zeros(1,n_nodes),0);

% calculate the eigenvalues (eig_values), the settling time (time_F) and   
% times vector (time) with a fixed sampling time (sample_int)
eig_val=eig(original_network);
tau=1/max(real(eig_val));
time_F=3*abs(tau);
sample_int=time_F/(time_points-1);
time=0:sample_int:time_F;

% initial condition of each state variable is chosen as a random value, 
% with a uniform distribution in [-1, 1]
% x0=-1 +2.*rand(n_nodes,1);

% define a vector that models the external perturbation
U=ones(1,length(time));

% Sample the state response
[Y,T,Meas_mtx] = lsim(sys,U,time,x0);


% add the desidered level of noise
noise=perc_noise*abs(Meas_mtx).*randn(time_points,n_nodes);
Meas_mtx_noise=noise+Meas_mtx;


if plot_data
    figure
    plot(Meas_mtx)
    figure
    plot(Meas_mtx_noise)   
end