function [original_network,B_vct,unstable_sys]=gen_ss_scale_free(Cnct_Mtx,eig_val_inf,eig_val_sup)

% This function generates a stable dynamical linear system 
% (dot(x)= Ax +Bu), whose A (dynamic matrix) has a structure corresponding 
% to the desired topology represented by the adjacency matrix Cnct_Mtx.
%
% INPUT VARIABLES
%
% Cnct_Mtx:    adjacency matrix with n_nodes*n_nodes elements
%
% eig_val_inf: limit inferior of the eigenvalues of the dynamic matrix
%
% eig_val_sup: limit superior of the eigenvalues of the dynamic matrix
%
% OUTPUT VARIABLES
%
% original_network:  the dynamic matrix of the stable dynamic linear system
%
% B_vct:             the vector that models the effect of external
%                    perturbations
% 
% unstable_sys:      if true the system is unstable
%


% number of nodes of the network
n_nodes=size(Cnct_Mtx,1);

% generate the stable linear system dot(x)=A x+ B u
sys_cont=rss(n_nodes,0,1);

% the elements of the dynamic Matrix (sys_full) and of the B vector (sys_b)
% are generally not null
sys_full=sys_cont.a;
sys_b=sys_cont.b;

% generate the B vector (B_vct), whose elements but one, randomly chosen
% are null
found_b=false;
while not(found_b)
    vec_perturb=randperm(n_nodes);
    one_pert=vec_perturb(1);
    if sys_b(one_pert)
        found_b=true;
    end
end
sys_b_one=zeros(n_nodes,1);
sys_b_one(one_pert)=1;
B_vct=sys_b.*sys_b_one;

% generate a system such that the eigenvalues have real part values
% uniformly distributed in the interval [eig_val_inf, eig_val_sup]
% the eigenvalues are shifted to the left half-plane by subtracting the 
% diagonal matrix (diag_sys_full)
diag_sys_full=eig_val_inf+(eig_val_sup-eig_val_inf).*rand(n_nodes,1);
sys_full=sys_full-diag(diag(sys_full))+diag(diag_sys_full);

% generate a scale-free stable system whoose dynamic matrix 
% (sys_scale_cont) has the topology of Cnct_Mtx
sys_scale_cont=sys_full.*Cnct_Mtx;

% check that the eigenvalues have real part values
% uniformly distributed in the interval [eig_val_inf, eig_val_sup],
% if not verified then shift further on the eigenvalues by subtracting an
% opportunity matrix (delta) to the sys_scale_cont matrix
eig_val=real(eig(sys_scale_cont));
Delta=zeros(n_nodes,n_nodes);
for idx=1:n_nodes
    if eig_val(idx)>eig_val_sup    
        Delta(idx,idx)=eig_val(idx)-eig_val_sup;
    elseif eig_val(idx)<eig_val_inf
        Delta(idx,idx)=eig_val(idx)-eig_val_inf;
    end   
end
original_network=sys_scale_cont-Delta;

% check the stability of the generated system whose dynamic Matrix is
% represented by original_network

if isempty(find(real(eig(original_network))>=0,1))
    unstable_sys=false;
else
    unstable_sys=true;
end

% eig_val_delta=(eig(original_network));
% unstable_sys=false;
% for idx=1:size(eig_val,1)
%     if real(eig_val_delta(idx))>=0
%         unstable_sys=true;
%     end
% end
