
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dr. Francesco Montefusco
%
% College of Engineering, Mathematics and Physical Sciences
% University of Exeter
% Harrison Building, North Park Road,
% Exeter EX4 4QF, UK.
% Email: F.Montefusco@exeter.ac.uk
%
% Biomechatronics Laboratory
% Universita' Magna Graecia di Catanzaro
% Campus Universitario "Salvatore Venuta"
% viale Europa, loc. Germaneto
% 88100 Catanzaro ITALY
% Tel:+39-0961-369-4196
% fax: +39-0961-369-4090
% e-mail: f.montefusco@unicz.it
% 
% This script allows to generate in silico networks, represented as
% dynamical linear system in the form dot(x)= Ax +Bu, whose A matrices 
% determines both the dynamics and the topology of the network. In 
% particular, the network topologies have been generated by using the 
% scalefree network generation model proposed by Albert and Barabasi.
% To this aim, the following functions are used:
% net_mask.m, gen_ss_scale_free.m, sel_constr.m, gen_data.m, 
% gen_data_noise.m
% The main parameters (prob_p, prop_q, n_link, etc.), that determine the 
% properties of the in silico networks, can be changed to edit in the 
% beginning of this file.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% DEFINE A SET OF PARAMETERS TO GENERATE THE NETWORKS %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc
close all

% define a part of the filename where your networks data are saved, for example:
% str_filename='data_net_in_silico_20n\sys_in_silico'
str_filename=input('digit part of the filename, where your networks are saved: ','s');

% number of nodes of the graph/network, we set to 20
n_nodes=20;

% nod_count: input variable of the function net_mask.m, we set it to 2, 3,
% 4, or 5 (nod_count<=n_link)
nod_count=2;

% n_link:input variable of the function net_mask.m, we set it to 1, 2 or 3
n_link=2;

% prob _p:input variable of the function net_mask.m, we set it to [0.4 0.8]
% according to the scalefree network generation model proposed by Albert 
% and Barabasi
prob_p=.6;

% prob_q:input variable of the function net_mask.m, we set it to [0 0.15]
% according to the scalefree network generation model proposed by Albert 
% and Barabasi
prob_q=.1;

% eig_val_inf: input variable of the function gen_ss_scale_free.m, we
% generally set it to -2,
eig_val_inf=-2;

% eig_val_sup: input variable of the function gen_ss_scale_free.m, we set
% it to -0.5
eig_val_sup=-0.5;

% number of generated networks
num_nets=input('how many networks, (if it is high, warning, a high number of plot, set plot data to false)');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%   GENERATION OF THE IN SILICO NETWORKS  %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for net=1:num_nets
    
    % generate a biological in silico network, the sparsity coefficient is 
    % in the interval [spar_coef_min spar_coef_max]; check the stability
    spar_coef=1;   
    spar_coef_min=0.65; % minimum value of the sparsity coefficent
    spar_coef_max=0.85; % maximum value of the sparsity coefficent
    unstable_sys=true; % flag true when the system is unstable 
    num_it=1; % iteration counter
    while spar_coef<=spar_coef_min || spar_coef>=spar_coef_max || unstable_sys 
        if num_it>20
            error('warning spar_coef, modify the spar_coef range, increase/ decrease n_link or prob_p')
        end
        % generation of the network topology
        Cnct_Mtx=net_mask(n_nodes,nod_count,n_link,prob_p,prob_q);
        % generation of a stable dynamical linear system in form
        % dot(x)=Ax+Bu
        % whose A matrix has a structure corresponding to the desired topology
        % determined by the connectivity matrix Cnct_Mtx'(the Cnct_Mtx transpose)  
        [original_network,B_vct,unstable_sys]=gen_ss_scale_free(Cnct_Mtx',eig_val_inf,eig_val_sup);
        n_elem_not_null=nnz(original_network);
        spar_coef=(n_nodes^2-n_elem_not_null)/(n_nodes^2-n_nodes);
        num_it=1+num_it;
    end

    % generation of the constraints matrices, with 10% and 20 % of
    % prior knowledge (PK), respectively
    Constr_mtx_10= sel_constr(original_network,0.1);
    Constr_mtx_20= sel_constr(original_network,0.2);


    % number of time points of the data set to genenerate
    time_points=25;


    plot_data=true; % flag true when we want to plot the data set,
                    % if you select a number of generated networks major
                    % than of five, it is preferible set this parameter to
                    % false
                    
    % calculate the state response of the generated biological in silico
    % network
    [Meas_mtx_25tp Meas_mtx_25tp_noise10 Meas_mtx_25tp_noise30]=...
        gen_data(original_network,B_vct,time_points,plot_data);

    
%     % if you want a different level of noise, you can run the function
%     % gen_data_noise
%     % level of noise
%         lev_noise=0.05;  
%         [Meas_mtx Meas_mtx_noise05]=gen_data_noise(original_network,...
%             B_vct,time_points,lev_noise,plot_data,Meas_mtx(1,:));

    clear spar_coef_min spar_coef_max unstable_sys plot_data n_elem_not_null num_it
    
    eval(['save ',str_filename,'_',num2str(n_nodes),'n_num_',num2str(net)]); 
end    